% use the training set to compute the correct weight for the new (angular 
% embedding) spb signal using the old (normalized cuts) spb signal as a 
% reference
function [w w2 w_vect w_vect2] = calibrate_spb(fg_data)
   % set type
   type = 'train';
   % set directories
   spb_dir  = ['~/spb_baseline/grouping/lib/out'];
   eigs_dir = ['../data/' type '/eigs'];
   % get number of images
   n_images = numel(fg_data);
   % initialize weights
   w_vect  = zeros([n_images 1]);
   w_vect2 = zeros([n_images 1]);
   % process images
   for n = 1:n_images
      % display status
      disp(['processing image ' num2str(n)]);
      % read image
      iid = fg_data(n).iid;
      im = imgRead(iid,'color');
      sx = size(im,1);
      sy = size(im,2);
      % load eigenvectors
      load([eigs_dir '/' num2str(iid) '_eigs.mat']);
      nvec = 16;
      % compute weights
      weights_ae = zeros(size(evals_ae));
      for v = 1:nvec
         if (abs(evals_ae(v)) > 0)
            weights_ae(v) = 1./sqrt(abs(evals_ae(v)));
         end
      end
      % sort by weight
      [junk inds] = sort(weights_ae);
      weights_ae = weights_ae(inds(end:-1:1));
      evecs_ae   = evecs_ae(:,inds(end:-1:1));
      % compute edges
      evecs = reshape(evecs_ae,[sx sy 1 nvec]);
      evecs = normalize_evecs(evecs);
      [er ero] = evecs_to_edges(real(evecs),weights_ae);
      [ei eio] = evecs_to_edges(imag(evecs),weights_ae);
      e  = sqrt(er.*er + ei.*ei);
      eo = sqrt(ero.*ero + eio.*eio);
      % compute edges (exclude first evec)
      evecs2 = reshape(evecs_ae(:,2:end),[sx sy 1 (nvec-1)]);
      evecs2 = normalize_evecs(evecs2);
      [er2 ero2] = evecs_to_edges(real(evecs2),weights_ae(2:end));
      [ei2 eio2] = evecs_to_edges(imag(evecs2),weights_ae(2:end));
      e2  = sqrt(er2.*er2 + ei2.*ei2);
      eo2 = sqrt(ero2.*ero2 + eio2.*eio2);
      % load normalized cuts spb signal
      spb = load([spb_dir '/' num2str(iid) '_pbs.mat']);
      spb = spb.sPb;
      % calibrate maximum
      m_spb = max(spb,[],3);
      m_eo  = max(eo,[],3);
      m_eo2 = max(eo2,[],3);
      % compute weights
      w_vect(n)  = mean(m_spb(:))./mean(m_eo(:));
      w_vect2(n) = mean(m_spb(:))./mean(m_eo2(:));
   end
   % compute average weight factors
   w  = mean(w_vect);
   w2 = mean(w_vect2);
end

function evecs = normalize_evecs(evecs)
   [sx sy sz nvec] = size(evecs);
   for n = 1:nvec
      a = abs(evecs(:,:,:,n));
      evecs(:,:,:,n) = evecs(:,:,:,n)./max(a(:));
   end
end
