% create detailed intermediate output for use in figures 
function create_example_figures(fg_data, fg_classifier, type, th)
   % default type - training data
   if (nargin < 3), type = 'train'; end
   % set directories
   pb_dir       = ['data/' type '/pb_components'];
   fg_dir       = ['data/' type '/fg_pb_corr'];
   examples_dir = ['data/examples'];
   % set gb parameters
   rs      = [4 6 8 11];
   nthetas = [16 16 16 16];
   alpha   = 0.5;
   beta_gb = 1;
   % repulsion radius for gb
   rrep = 3;
   % maximum number of descriptors per image
   n_samp = 10.*1000;
   % threshold on for sampling nonmax suppressed pb
   nmax_thresh = 0.3;
   % connection radius
   radius = 5;
   % weight on ordering
   w_order = 0.0001;
   % get number of images
   n_images = numel(fg_data);
   % predict segmentation and f/g labels
   for n = 1:n_images
      % read image
      iid = fg_data(n).iid;
      im = imgRead(iid,'color');
      % compute multiscale pb
      if (~exist([pb_dir '/' num2str(iid) '.mat'],'file'))
         %% compute cues - standard scales
         [bg1 bg2 bg3 cga1 cga2 cga3 cgb1 cgb2 cgb3 tg1 tg2 tg3 textons] = det_mPb(im);
         %% smooth cues
         tic;
         gtheta = [1.5708    1.1781    0.7854    0.3927   0    2.7489    2.3562    1.9635];
         for o = 1 : size(tg1, 3),
             bg1(:,:,o) = fitparab(bg1(:,:,o),3,3/4,gtheta(o));
             bg2(:,:,o) = fitparab(bg2(:,:,o),5,5/4,gtheta(o));
             bg3(:,:,o) = fitparab(bg3(:,:,o),10,10/4,gtheta(o));

             cga1(:,:,o) = fitparab(cga1(:,:,o),5,5/4,gtheta(o));
             cga2(:,:,o) = fitparab(cga2(:,:,o),10,10/4,gtheta(o));
             cga3(:,:,o) = fitparab(cga3(:,:,o),20,20/4,gtheta(o));

             cgb1(:,:,o) = fitparab(cgb1(:,:,o),5,5/4,gtheta(o));
             cgb2(:,:,o) = fitparab(cgb2(:,:,o),10,10/4,gtheta(o));
             cgb3(:,:,o) = fitparab(cgb3(:,:,o),20,20/4,gtheta(o));

             tg1(:,:,o) = fitparab(tg1(:,:,o),5,5/4,gtheta(o));
             tg2(:,:,o) = fitparab(tg2(:,:,o),10,10/4,gtheta(o));
             tg3(:,:,o) = fitparab(tg3(:,:,o),20,20/4,gtheta(o));
         end
         fprintf('Cues smoothing:%g\n', toc);
         % save result
         save([pb_dir '/' num2str(iid) '.mat'],'bg1','bg2','bg3','cga1','cga2','cga3','cgb1','cgb2','cgb3','tg1','tg2','tg3');
      end
      % read pb components
      pb = load([pb_dir '/' num2str(iid) '.mat']);
      pb_no_rescale = mPb_all_from_cues_no_rescale( ...
         pb.bg1, pb.bg2, pb.bg3, ...
         pb.cga1, pb.cga2, pb.cga3, ...
         pb.cgb1, pb.cgb2, pb.cgb3, ...
         pb.tg1, pb.tg2, pb.tg3, 1.0);
      pb = mPb_all_from_cues( ...
         pb.bg1, pb.bg2, pb.bg3, ...
         pb.cga1, pb.cga2, pb.cga3, ...
         pb.cgb1, pb.cgb2, pb.cgb3, ...
         pb.tg1, pb.tg2, pb.tg3, 1.0);
      [pbm ori] = max(pb,[],3);
      [sx sy] = size(pbm);
      % compute nonmax suppressed pb
      nmax = nonmax_channels(pb_no_rescale,pi/2);
      nmax = max(0, min(1, 1.2*nmax));
      % sample new descriptor locations
      [ci cj] = find_interest_points(nmax.*(nmax > nmax_thresh), n_samp, rrep);
      n_curr = numel(ci);
      inds = sub2ind([sx sy],ci,cj);
      % compute gb descriptors
      [descriptors sample_points] = ...
         compute_gb(nmax,ci,cj,alpha,beta_gb,rs,nthetas);
      % normalize descriptors
      descriptors = normalize_descriptors(descriptors,rs,nthetas);
      % rotate descriptors
      drot = zeros(size(descriptors));
      for idx = 1:n_curr
         frot = reshape(descriptors(idx,:),[16 4]);
         frot = circshift(frot,[(ori(inds(idx))-1) 0]);
         drot(idx,:) = reshape(frot,[1 64]);
      end
      % compute squared distance from each descriptor to each cluster center
      k = size(fg_classifier.shape_centroids,1);
      dmx = zeros([n_curr k]);
      for idx = 1:n_curr
         dist = repmat(drot(idx,:),[k 1]) - fg_classifier.shape_centroids;
         dist = sum(dist.*dist,2);
         dmx(idx,:) = dist;
      end
      % form feature matrix
      F = [exp(-dmx./(fg_classifier.sigma)) ones([n_curr 1])];
      % predict f/g
      preds = 1./(1+exp(-F*(fg_classifier.beta)));
      % show predictions
      [xcs ycs] = ind2sub([sx sy],inds);
      n_ori = 8;
      oris = (0:(n_ori-1))/n_ori * pi;
      os = (oris(ori(inds)) + pi/2).';
      os = os + pi.*(preds < 0.5);
      xcs_a = xcs + (radius).*abs(preds-0.5).*8.*cos(os).*nmax(inds);
      ycs_a = ycs + (radius).*abs(preds-0.5).*8.*sin(os).*nmax(inds);
      %figure(1); imagesc(nmax); axis image; colormap(gray);
      %hold on;
      %plot([ycs ycs_a].',[xcs xcs_a].','g');
      %plot([ycs_a].',[xcs_a].','w.');
      %hold off;
      %drawnow;
      % construct links
      xa = round(xcs + (radius).*cos(os));
      ya = round(ycs + (radius).*sin(os));
      xb = round(xcs - (radius).*cos(os));
      yb = round(ycs - (radius).*sin(os));
      g = find( ...
         (xa > 1) & (xa < sx) & (xb > 1) & (xb < sx) & ...
         (ya > 1) & (ya < sy) & (yb > 1) & (yb < sy));
      ia = sub2ind([sx sy],xa(g),ya(g));
      ib = sub2ind([sx sy],xb(g),yb(g));
      vals = repmat(pi./8,[numel(ia) 1]);
      conf = 2.*abs(preds(g)-0.5).*nmax(inds(g));
      index_a = cat(1, ia, ib);
      index_b = cat(1, ib, ia);
      vals_all = cat(1, vals, -vals);
      conf_all = cat(1, conf, conf);
      % construct affinity matrix
      rho = 0.1;
      A = pb_to_affinity(pb, rho);
      % create sparse ordering matrices
      [unique_idx u] = unique([index_a index_b],'rows');
      R   = sparse(index_a(u), index_b(u), vals_all(u), (sx*sy), (sx*sy));
      Cmx = sparse(index_a(u), index_b(u), conf_all(u), (sx*sy), (sx*sy));
      % compute eigenvectors for simulataneous grouping and f/g problem
      nvec = 16;
      [evecs_ae evals_ae] = ncut_ae(A + w_order.*Cmx, R, nvec);
      % compute weights
      weights_ae = zeros(size(evals_ae));
      for v = 1:nvec
         if (abs(evals_ae(v)) > 0)
            weights_ae(v) = 1./sqrt(abs(evals_ae(v)));
         end
      end
      % sort by weight
      [junk inds] = sort(weights_ae);
      weights_ae = weights_ae(inds(end:-1:1));
      evecs_ae   = evecs_ae(:,inds(end:-1:1));
      % reshape evecs 
      evecs = reshape(evecs_ae,[sx sy 1 nvec]);
      evecs = normalize_evecs(evecs);
      % compute edges (exclude first evec)
      evecs2 = reshape(evecs_ae(:,2:end),[sx sy 1 (nvec-1)]);
      evecs2 = normalize_evecs(evecs2);
      [er2 ero2] = evecs_to_edges(real(evecs2),weights_ae(2:end));
      [ei2 eio2] = evecs_to_edges(imag(evecs2),weights_ae(2:end));
      e2  = sqrt(er2.*er2 + ei2.*ei2);
      eo2 = sqrt(ero2.*ero2 + eio2.*eio2);
      % reload pb components
      pb = load([pb_dir '/' num2str(iid) '.mat']);
      % compute gpb for these edges
      ae_pb_orient2 = gPb_from_cues( ...
         pb.bg1, pb.bg2, pb.bg3, ...
         pb.cga1, pb.cga2, pb.cga3, ...
         pb.cgb1, pb.cgb2, pb.cgb3, ...
         pb.tg1, pb.tg2, pb.tg3, 0.0010.*eo2); % weight learned from calibrate_spb
      % enforce nonnegativity
      ae_pb_orient2 = max(0,ae_pb_orient2);
      % compute large ucms
      ucm_ae2_lg = double(contours2ucm(ae_pb_orient2.*255,'doubleSize'))./255;
      % compute ucms
      ucm_ae2 = ucm_ae2_lg(3:2:end, 3:2:end);
      % compute per-pixel figure/ground
      fg_ae = reshape(angle(evecs_ae(:,1)),[sx sy]);
      % remap figure/ground to 3 std dev from mean
      fg_m   = mean(fg_ae(:));
      fg_std = std(fg_ae(:));
      ulimit = fg_m + 3.*fg_std;
      llimit = fg_m - 3.*fg_std;
      fg_ae(find(fg_ae > ulimit)) = ulimit;
      fg_ae(find(fg_ae < llimit)) = llimit;
      fg_ae = (fg_ae - min(fg_ae(:)))./(max(fg_ae(:))-min(fg_ae(:)) + eps);
      % segment at fixed threshold
      S = segment_ucm(ucm_ae2_lg.*(ucm_ae2_lg>th));
      % average fg over regions at a fixed threshold
      fg_ae_avg = zeros(size(fg_ae));
      for s = 1:(S.n_regions)
         fg_ae_avg(S.seg_members{s}) = mean(fg_ae(S.seg_members{s}));
      end
      fg_ae_avg = (fg_ae_avg - min(fg_ae_avg(:)))./(max(fg_ae_avg(:))-min(fg_ae_avg(:)) + eps);
      % average image over regions at fixed threshold
      im_r = im(:,:,1);
      im_g = im(:,:,2);
      im_b = im(:,:,3);
      im_r_avg = zeros(size(im_r));
      im_g_avg = zeros(size(im_g));
      im_b_avg = zeros(size(im_b));
      for s = 1:(S.n_regions)
         im_r_avg(S.seg_members{s}) = mean(im_r(S.seg_members{s}));
         im_g_avg(S.seg_members{s}) = mean(im_g(S.seg_members{s}));
         im_b_avg(S.seg_members{s}) = mean(im_b(S.seg_members{s}));
      end
      im_avg = cat(3,im_r_avg,im_g_avg,im_b_avg);
      % create output directory
      output_dir = [examples_dir '/' num2str(iid)];
      mkdir(output_dir);
      % create output
      imwrite(im,        [output_dir '/' 'im.png']);
      imwrite(1-ucm_ae2, [output_dir '/' 'ucm.png']);
      imwrite(ind2rgb(round(fg_ae.*255)+1,jet(256)), ...
         [output_dir '/' 'fg.png']);
      imwrite(ind2rgb(round(fg_ae_avg.*255)+1,jet(256)), ...
         [output_dir '/' 'fg_seg.png']);
      imwrite(im_avg,[output_dir '/' 'seg.png']);
      % create local figure/ground output
      inds = sub2ind([sx sy],ci,cj);
      xcs_a = xcs + 2.*(radius).*abs(preds-0.5).*8.*cos(os).*nmax(inds);
      ycs_a = ycs + 2.*(radius).*abs(preds-0.5).*8.*sin(os).*nmax(inds);
      figure(10); imagesc(1-nmax); axis image; colormap(gray);
      hold on;
      plot([ycs ycs_a].',[xcs xcs_a].','g','LineWidth',1);
      plot([ycs_a].',[xcs_a].','r.');
      set(10,'Position',[500 500 960 640]);
      set(gca,'XTick',[]);
      set(gca,'YTick',[]);
      whitebg(10,'w');
      print(10,'-depsc',[output_dir '/' 'fg_local.eps']);
      % compute evec edges
      e_edges = zeros([sx sy nvec]);
      for v = 1:nvec
         e_edge_r = evecs_to_edges(real(evecs(:,:,:,v)),1);
         e_edge_i = evecs_to_edges(imag(evecs(:,:,:,v)),1);
         e_edges(:,:,v) = sqrt(e_edge_r.*e_edge_r + e_edge_i.*e_edge_i);
      end
      % write evecs
      for v = 1:nvec
         e_r = real(evecs(:,:,:,v));
         e_i = imag(evecs(:,:,:,v));
         e_r = (e_r - min(e_r(:)))./(max(e_r(:))-min(e_r(:))+eps);
         e_i = (e_i - min(e_i(:)))./(max(e_i(:))-min(e_i(:))+eps);
         imwrite(e_r,[output_dir '/' 'evec_' num2str(v-1,'%02d') '_real.png']);
         imwrite(e_i,[output_dir '/' 'evec_' num2str(v-1,'%02d') '_imag.png']);
      end
      % write out combined evecs
      for v = 1:nvec
         e_r = real(evecs(:,:,:,v));
         e_i = imag(evecs(:,:,:,v));
         e_r = (e_r - min(e_r(:)))./(max(e_r(:))-min(e_r(:))+eps);
         e_i = (e_i - min(e_i(:)))./(max(e_i(:))-min(e_i(:))+eps);
         imwrite(cat(1,e_r,e_i),[output_dir '/' 'evec_' num2str(v-1,'%02d') '.png']);
      end
      % write edges
      for v = 2:nvec
         imwrite(1-e_edges(:,:,v),[output_dir '/' 'edge_' num2str(v-1,'%02d') '.png']);
      end
      % write out pb
      imwrite(ind2rgb(round(pbm./max(pbm(:)).*255)+1,jet(256)), [output_dir '/' 'pb.png']);
      imwrite(1-pbm./max(pbm(:)), [output_dir '/' 'pb_grayscale.png']);
      % write out spb
      imwrite(1-e2./max(e2(:)), [output_dir '/' 'spb_ae.png']);
      imwrite(ind2rgb(round((e2./max(e2(:))).*255+1),jet(256)), [output_dir '/' 'spb_ae_color.png']);
   end
end

function evecs = normalize_evecs(evecs)
   [sx sy sz nvec] = size(evecs);
   for n = 1:nvec
      a = abs(evecs(:,:,:,n));
      evecs(:,:,:,n) = evecs(:,:,:,n)./max(a(:));
   end
end
